@@ -7,6 +7,7 @@ use namespace::autoclean;
 
 use B::Hooks::EndOfScope;
 use Moose 0.90 ();
+use Moose::Util::MetaRole;
 
 # debugging
 #use Smart::Comments '###', '####';
@@ -17,7 +18,7 @@ MooseX::MarkAsMethods - Mark overload code symbols as methods
 
 =cut
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 =head1 SYNOPSIS
 
@@ -36,6 +37,9 @@ our $VERSION = '0.06';
     # overloads defined in a role will "just work" when the role is
     # composed into a class
 
+    # additional methods generated outside Class::MOP/Moose can be marked, too
+    __PACKAGE__->meta->mark_as_method('foo');
+
     package Bar;
     use Moose;
 
@@ -48,8 +52,8 @@ our $VERSION = '0.06';
 =head1 DESCRIPTION
 
 MooseX::MarkAsMethods allows one to easily mark certain functions as Moose
-methods.  This will allow other packages such as namespace::autoclean to
-operate without, say, blowing away your overloads.  After using
+methods.  This will allow other packages such as L<namespace::autoclean> to
+operate without blowing away your overloads.  After using
 MooseX::MarkAsMethods your overloads will be recognized by L<Class::MOP> as
 being methods, and class extension as well as composition from roles with
 overloads will "just work".
@@ -59,7 +63,42 @@ By default we check for overloads, and mark those functions as methods.
 If 'autoclean => 1' is passed to import on use'ing this module, we will invoke
 namespace::autoclean to clear out non-methods.
 
-=head1 CAVEAT
+=head1 TRAITS APPLIED
+
+use'ing this package causes a trait to be applied to your metaclass (for both
+roles and classes), that provides a mark_as_method() method.  You can use this
+to mark newly generated methods at runtime (e.g. during class composition)
+that some other package has created for you.
+
+mark_as_method() is invoked with one or more names to mark as a method.  We die
+on any error (e.g. name not in symbol table, already a method, etc).  e.g.
+
+    __PACKAGE__->meta->mark_as_method('newly_generated');
+
+e.g. say you have some sugar from another package that creates accessors of
+some sort; you could mark them as methods via a method modifier:
+
+    # called as __PACKAGE__->foo_generator('name', ...)
+    after 'foo_generator' => sub {
+
+        shift->meta->mark_as_method(shift);
+    };
+
+=head1 IMPLICATIONS FOR ROLES
+
+Using MooseX::MarkAsMethods in a role will cause Moose to track and treat your
+overloads like any other method defined in the role, and things will "just
+work".  That's it.
+
+=head1 CAVEATS
+
+=head2 meta->mark_as_method()
+
+B<You almost certainly don't need or want to do this.>  CMOP/Moose are fairly
+good about determining what is and what isn't a method, but not perfect.
+Before using this method, you should pause and think about why you need to.
+
+=head2 namespace::autoclean
 
 As currently implemented, we run our "method maker" at the end of the calling
 package's compile scope (L<B::Hooks::EndOfScope>).  As L<namespace::autoclean>
@@ -85,31 +124,73 @@ to worry about ordering is:
 =cut
 
 {
-    package MooseX::MarkAsMethods::Meta::Method::Overload;
+    package MooseX::MarkAsMethods::MetaRole::MethodMarker;
+    use Moose::Role;
     use namespace::autoclean;
 
-    use base 'Moose::Meta::Method';
+    sub mark_as_method {
+        my $self = shift @_;
 
-    our $VERSION = '0.06';
+        $self->_mark_as_method($_) for @_;
+        return;
+    }
 
-    # strictly speaking, we don't need to do this; we could just use
-    # Moose::Meta::Method or even Class::MOP::Method...  But it might be
-    # useful to easily differentiate these added methods.
+    sub _mark_as_method {
+        my ($self, $method_name) = @_;
+
+        do { warn "$method_name is already a method!"; return }
+            if $self->has_method($method_name);
+
+        my $code = $self->get_package_symbol({
+            name  => $method_name,
+            sigil => '&',
+            type  => 'CODE',
+        });
+
+        do { warn "$method_name not found as a CODE symbol!"; return }
+            unless defined $code;
+
+        $self->add_method($method_name =>
+            $self->wrap_method_body(
+                associated_metaclass => $self,
+                name => $method_name,
+                body => $code,
+            ),
+        );
+
+        return;
+    }
+}
+
+sub init_meta {
+    my ($class, %options) = @_;
+    my $for_class = $options{for_class};
+
+    Moose::Util::MetaRole::apply_metaroles(
+        for => $for_class,
+        class_metaroles => {
+           class => ['MooseX::MarkAsMethods::MetaRole::MethodMarker'],
+        },
+        role_metaroles => {
+            role => ['MooseX::MarkAsMethods::MetaRole::MethodMarker'],
+        },
+    );
+
+    return $for_class->meta;
 }
 
 sub import {
     my ($class, %args) = @_;
 
-    # our invoking package
     my $target = scalar caller;
+    return if $target eq 'main';
+    $class->init_meta(for_class => $target);
 
     on_scope_end {
 
         my $meta = Class::MOP::Class->initialize($target);
 
         ### metaclass: ref $meta
-        return unless $meta && ref $meta ne 'Class::MOP::Class';
-
         my %methods   = map { ($_ => 1) } $meta->get_method_list;
         my %symbols   = %{ $meta->get_all_package_symbols('CODE') };
         my @overloads = grep { /^\(/ } keys %symbols;
@@ -123,14 +204,7 @@ sub import {
             next if $methods{$overload_name};
 
             ### marking as method: $overload_name
-            my $method = MooseX::MarkAsMethods::Meta::Method::Overload->wrap(
-                associated_metaclass => $meta,
-                package_name         => $target,
-                name                 => $overload_name,
-                body                 => $symbols{$overload_name},
-            );
-
-            $meta->add_method($overload_name => $method);
+            $meta->mark_as_method($overload_name);
             $methods{$overload_name} = 1;
             delete $symbols{$overload_name};
         }
